#/*++
#  Copyright (c) 2008, Intel Corporation
#  All rights reserved. 
# 
#  This UEFI SCT ("Software") is furnished under license and may only be used or copied
#  in accordance with the terms of that license UEFI Contributor Agreement.
#  No license, express or implied, by estoppel or otherwise, to any intellectual property
#  rights is granted by this document. The Software is subject to change without notice,
#  and should not be construed as a commitment by Intel Corporation to market, license,
#  sell or support any product or technology. Unless otherwise provided for in the license
#  under which this Software is provided, the Software is provided AS IS, with no warranties
#  of any kind, express or implied. Except as expressly permitted by the Software license,
#  neither Intel Corporation nor its suppliers assumes any responsibility or liability
#  for any errors or inaccuracies that may appear herein.
#--*/
################################################################################
CaseLevel         CONFORMANCE
CaseAttribute     AUTO
CaseVerboseLevel  DEFAULT
set reportfile    report.csv

#
# test case Name, category, description, GUID...
#
CaseGuid        9775E91C-A85D-42e4-8959-976455A506F0
CaseName        CreateChild.Conf1.Case1
CaseCategory    MNP
CaseDescription {Test the CreateChild conformance of MNPSB - Call              \
	               MnpServiceBinding.CreateChild() with the parameter ChildHandle\
	               being NULL. The return status should be EFI_INVALID_PARAMETER.}
################################################################################

Include MNP/include/Mnp.inc.tcl

#
# Begin log ...
#
BeginLog
BeginScope _MNP_CREATECHILD_CONFORMANCE1_CASE1_

#
# Parameter Definition
# R_ represents "Remote EFI Side Parameter"
# L_ represents "Local OS Side Parameter"
#
UINTN                            R_Status
UINTN                            R_Handle

SetVar  R_Handle   0

#
# The pointer is NULL, return EFI_INVALID_PARAMETER
#
MnpServiceBinding->CreateChild "NULL, &@R_Status"
GetAck
set assert [VerifyReturnStatus R_Status $EFI_INVALID_PARAMETER]
RecordAssertion $assert $MnpCreateChildConf1AssertionGuid001                   \
                "MnpSB.CreateChild - Create Child 1"                           \
                "ReturnStatus - $R_Status,ExpectedStatus - $EFI_INVALID_PARAMETER"


#
# The pointer is not NULL, return EFI_SUCCESS
#
MnpServiceBinding->CreateChild "&@R_Handle, &@R_Status"
GetAck
set assert [VerifyReturnStatus R_Status $EFI_SUCCESS]
RecordAssertion $assert $MnpCreateChildConf1AssertionGuid002                   \
                "MnpSB.CreateChild - Create Child 1 - Install I/0"             \
                "ReturnStatus - $R_Status,ExpectedStatus - $EFI_SUCCESS"                             

MnpServiceBinding->DestroyChild "@R_Handle, &@R_Status"
GetAck
set assert [VerifyReturnStatus R_Status $EFI_SUCCESS]
RecordAssertion $assert $GenericAssertionGuid                                  \
                "MnpSB.CreateChild - Destroy Child 1"                          \
                "ReturnStatus - $R_Status,ExpectedStatus - $EFI_SUCCESS"
                            

EndScope      _MNP_CREATECHILD_CONFORMANCE1_CASE1_

#
# End log
#
EndLog
