/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:
  
    EmsPktSend.c
    
Abstract:

    

--*/

#include "EmsTclInit.h"
#include "EmsNet.h"
#include "EmsPktSend.h"
#include "EmsUtilityString.h"
#include "EmsLogCommand.h"

STATIC Tcl_CmdProc  TclSendPacket;

VOID_P
SendTclInit (
  IN Tcl_Interp *Interp
  )
/*++

Routine Description:

  Packet sending related TCL initialization routine.

Arguments:

  Interp      - TCL intepreter.

Returns:

  None

--*/
{
  //
  // Create TCL commands here
  //
  Tcl_CreateCommand (
    Interp,
    "SendPacket",
    TclSendPacket,
    (ClientData) NULL,
    (Tcl_CmdDeleteProc *) NULL
    );

}

STATIC
INT32
TclSendPacket (
  IN ClientData        clientData,
  IN Tcl_Interp        *Interp,
  IN INT32             Argc,
  IN CONST84 INT8      *Argv[]
  )
/*++

Routine Description:

  TCL command "SendPacket" implementation routine

Arguments:

  clientData  - Private data, if any.
  Interp      - TCL intepreter.
  Argc        - Argument counter.
  Argv        - Argument value pointer array.

Returns:

  TCL_OK or TCL_ERROR

--*/
{
  INT8      *Name;
  libnet_t  *l;
  INT32     Repeat;
  INT32     Count;
  INT32     Index;
  INT8      Result[100];

  LogCurrentCommand (Argc, Argv);
  //
  // sendpacket packetname
  //
  if ((Argc != 2) && (Argc != 4)) {
    goto FormatWrong;
  }

  Repeat  = 1;

  Name    = (INT8 *) Argv[1];
  if (Name[0] == '-') {
    goto FormatWrong;
  }

  if (4 == Argc) {
    if (0 != strcmp_i ((INT8 *) Argv[2], "-c")) {
      goto FormatWrong;
    }

    if ((AsciiStringToUint32 ((INT8 *) Argv[3], &Repeat) <= 0) || (0 == Repeat)) {
      goto FormatWrong;
    }
  }

  l = libnet_cq_find_by_label (Name);
  if (NULL == l) {
    goto NoSuchPacket;
  }

  Count = 0;
  for (Index = 0; Index < Repeat; Index++) {
    if (-1 == libnet_write (l)) {
      continue;
    };
    Count++;
  }

  sprintf (Result, "%d", Count);
  //
  //  Tcl_AppendResult(interp, result, (char *) NULL);
  //
  return TCL_OK;

FormatWrong:
  //
  //  Tcl_AppendResult(interp, "SendPacket: SendPacket packetname [-c repeat] \n", (char *) NULL);
  //
  return TCL_ERROR;

NoSuchPacket:
  //
  //  Tcl_AppendResult(interp, "SendPacket: The specified packet not found \n", (char *) NULL);
  //
  return TCL_ERROR;
}
