/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:
  
    EmsTimer.h
    
Abstract:

    Incude header files for EMS TCL Timer functions

--*/

#ifndef __EMSTIMER__
#define __EMSTIMER__

#include <tcl.h>
#include <EmsTypes.h>

typedef struct EmsTimerStruct {
  UINT32  Timeout;
  BOOLEAN TimerValid;
  VOID (*Func) (VOID *Arg);
  VOID  *Arg;
} EmsTimer;

VOID
EmsDelTimer (
  EmsTimer *Timer
  )
/*++

Routine Description:

  The implementation of deleting an EMS timer

Arguments:

  Timer - The EMS timer to be deleted

Returns:

  None

--*/
;

EmsTimer  *
EmsSetTimer (
  UINT32     Timeout,
  VOID (*HandleTimeout) (VOID *),
  VOID *Arg
  )
/*++

Routine Description:

  The implementation of creating an EMS timer

Arguments:

  Timeout        - The maximum seconds the timer will wait
  HandleTimeout  - The callback function if timer timeout
  Arg            - The parameter of callback function

Returns:

  The new EMS timer

--*/
;

#endif
