/*++
 Copyright (c) 2008, Byosoft.NJ
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:

  AbsolutePointerProtocol.h

Abstract:

  Absolute Pointer protocol as defined in the UEFI 2.1 specification.

--*/


#ifndef _ABSOLUTE_POINTER_FOR_TEST_H_
#define _ABSOLUTE_POINTER_FOR_TEST_H_

//
// Global ID for the Absolute Pointer Protocol
//
#define EFI_ABSOLUTE_POINTER_PROTOCOL_GUID \
    { 0x8d5932b, 0xc655, 0x4ae9, { 0x9b, 0x15, 0xf2, 0x59, 0x04, 0x99, 0x2a, 0x43} }



EFI_FORWARD_DECLARATION(EFI_ABSOLUTE_POINTER_PROTOCOL);



//******************************************************* 
// EFI_ABSOLUTE_POINTER_STATE 
//******************************************************* 
typedef struct { 
  UINT64 CurrentX; 
  UINT64 CurrentY; 
  UINT64 CurrentZ; 
  UINT32 ActiveButtons; 
} EFI_ABSOLUTE_POINTER_STATE;

//****************************** 
//definitions of bits within ActiveButtons 
//****************************** 
#define EFI_ABSP_TouchActive                  0x00000001 
#define EFI_ABS_AltActive                     0x00000002

//******************************************************* 
// EFI_ABSOLUTE_POINTER_MODE 
//******************************************************* 
typedef struct {
  UINT64 AbsoluteMinX; 
  UINT64 AbsoluteMinY; 
  UINT64 AbsoluteMinZ; 
  UINT64 AbsoluteMaxX; 
  UINT64 AbsoluteMaxY; 
  UINT64 AbsoluteMaxZ; 
  UINT32 Attributes; 
} EFI_ABSOLUTE_POINTER_MODE; 

#define EFI_ABSP_SupportsAltActive                       0x00000001
#define EFI_ABSP_SupportsPressureAsZ                     0x00000002


//
// Prototypes for the Absolute Pointer Protocol
//

typedef
EFI_STATUS
(EFIAPI *EFI_ABSOLUTE_POINTER_RESET) (
  IN  EFI_ABSOLUTE_POINTER_PROTOCOL             *This,
  IN  BOOLEAN                                   ExtendedVerification
  );

typedef
EFI_STATUS
(EFIAPI *EFI_ABSOLUTE_POINTER_GET_STATE) (
   IN EFI_ABSOLUTE_POINTER_PROTOCOL               *This, 
   IN OUT EFI_ABSOLUTE_POINTER_STATE               *State 
   );

//
// Interface structure for the Absolute Pointer Protocol
//
typedef struct _EFI_ABSOLUTE_POINTER_PROTOCOL {
  EFI_ABSOLUTE_POINTER_RESET       Reset;
  EFI_ABSOLUTE_POINTER_GET_STATE    GetState;
  EFI_EVENT                         WaitForInput;
  EFI_ABSOLUTE_POINTER_MODE         *Mode;
} EFI_ABSOLUTE_POINTER_PROTOCOL;

extern EFI_GUID gEfiAbsolutePointerProtocolGuid;



#endif /* _ABSOLUTE_POINTER_FOR_TEST_H_ */
