/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:

  LibPrivate.h

Abstract:

  This file defines the private interfaces of the test support libraries. Only
  the test management system can invokes these private interfaces to initialize
  and configure the test support libraries.

--*/

#ifndef _EFI_LIB_PRIVATE_H_
#define _EFI_LIB_PRIVATE_H_

//
// Includes
//
#include "Efi.h"
#include "EfiTest.h"
#include "LibConfig.h"
#include EFI_TEST_PROTOCOL_DEFINITION (TslInit)

//
// Private interface for each test support library
//

#define TSL_INIT_PRIVATE_DATA_SIGNATURE     EFI_SIGNATURE_32('T','S','L','I')

typedef struct {
  UINT32                              Signature;
  EFI_HANDLE                          ImageHandle;
  EFI_TSL_INIT_INTERFACE              TslInit;
} TSL_INIT_PRIVATE_DATA;

#define TSL_INIT_PRIVATE_DATA_FROM_THIS(a)  \
  CR(a, TSL_INIT_PRIVATE_DATA, TslInit, TSL_INIT_PRIVATE_DATA_SIGNATURE)

//
// Standard test library's private interface
//

EFI_FORWARD_DECLARATION(EFI_STANDARD_TSL_PRIVATE_INTERFACE);

typedef
EFI_STATUS
(EFIAPI * EFI_STSL_SET_CONFIG) (
  IN EFI_STANDARD_TSL_PRIVATE_INTERFACE     *This,
  IN EFI_LIB_CONFIG_DATA                    *Config
  );

typedef
EFI_STATUS
(EFIAPI * EFI_STSL_BEGIN_LOGGING) (
  IN EFI_STANDARD_TSL_PRIVATE_INTERFACE     *This
  );

typedef
EFI_STATUS
(EFIAPI * EFI_STSL_END_LOGGING) (
  IN EFI_STANDARD_TSL_PRIVATE_INTERFACE     *This,
  IN EFI_STATUS                             TestStatus
  );

typedef struct _EFI_STANDARD_TSL_PRIVATE_INTERFACE {
  EFI_STSL_SET_CONFIG                       SetConfig;
  EFI_STSL_BEGIN_LOGGING                    BeginLogging;
  EFI_STSL_END_LOGGING                      EndLogging;
} EFI_STANDARD_TSL_PRIVATE_INTERFACE;

//
// Test logging library's private interface
//

EFI_FORWARD_DECLARATION(EFI_TLL_PRIVATE_INTERFACE);

typedef
EFI_STATUS
(EFIAPI * EFI_TLL_SET_CONFIG) (
  IN EFI_TLL_PRIVATE_INTERFACE              *This,
  IN EFI_LIB_CONFIG_DATA                    *Config
  );

typedef
EFI_STATUS
(EFIAPI * EFI_TLL_BEGIN_LOGGING) (
  IN EFI_TLL_PRIVATE_INTERFACE              *This
  );

typedef
EFI_STATUS
(EFIAPI * EFI_TLL_END_LOGGING) (
  IN EFI_TLL_PRIVATE_INTERFACE              *This,
  IN EFI_STATUS                             TestStatus
  );

typedef struct _EFI_TLL_PRIVATE_INTERFACE {
  EFI_TLL_SET_CONFIG                        SetConfig;
  EFI_TLL_BEGIN_LOGGING                     BeginLogging;
  EFI_TLL_END_LOGGING                       EndLogging;
} EFI_TLL_PRIVATE_INTERFACE;

//
// Test recovery library's private interface
//

EFI_FORWARD_DECLARATION(EFI_TRL_PRIVATE_INTERFACE);

typedef
EFI_STATUS
(EFIAPI * EFI_TRL_SET_CONFIG) (
  IN EFI_TRL_PRIVATE_INTERFACE              *This,
  IN EFI_DEVICE_PATH_PROTOCOL               *DevicePath,
  IN CHAR16                                 *FileName
  );

typedef struct _EFI_TRL_PRIVATE_INTERFACE {
  EFI_TRL_SET_CONFIG                        SetConfig;
} EFI_TRL_PRIVATE_INTERFACE;

#endif
