/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:

  EasCmdDisp.h

Abstract:

  SCT extended data services declarations.

--*/

#ifndef _EAS_CMD_DISP_H_
#define _EAS_CMD_DISP_H_

//
// External functions declarations
//
EFI_STATUS
PreDispatch (
  IN  CHAR16               *Buffer
  )
/*++

Routine Description:

  Pre command dispatch.

Arguments:

  Buffer  - Command buffer.

Returns:

  EFI_SUCCESS - Operation succeeded.
  Others      - Some failure happened.

--*/
;

EFI_STATUS
PostDispatch (
  IN EFI_MONITOR_COMMAND           *Cmd
  )
/*++

Routine Description:

  Post command dispatch.

Arguments:

  None

Returns:

  EFI_SUCCESS - Operation succeeded.
  Others      - Some failure happened.
  
--*/
;

EFI_STATUS
GetCmdDispatch (
  IN  CHAR16               *Buffer,
  OUT EFI_MONITOR_COMMAND  *Cmd
  );

EFI_STATUS
DispatchInternalComd (
  VOID
  )
/*++

Routine Description:

  Dispatch internal command

Arguments:

  None

Returns:

  EFI_SUCCESS - Operation succeeded.
  EFI_OUT_OF_RESOURCES - Memory allocation failed.
  Others      - Some failure happened.

--*/
;

EFI_STATUS
DispatchExecComd (
  VOID
  )
/*++

Routine Description:

  Dispatch an exec command

Arguments:

  None

Returns:

  EFI_SUCCESS - Operation succeeded.
  EFI_OUT_OF_RESOURCES - Memory allocation failed.
  Others      - Some failure happened.

--*/
;

EFI_STATUS
DispatchEletComd (
  VOID
  )
/*++

Routine Description:

  Dispatch an elet command.

Arguments:

  None

Returns:

  EFI_SUCCESS - Operation succeeded.
  Others      - Some failure happened.

--*/
;

EFI_STATUS
DispatchFileTransferComd (
  ENTS_CMD_TYPE FileCmdType
  )
/*++

Routine Description:

  Dispatch an get_file command, download file from SCT management side.

Arguments:

  FileCmdType - File command type.

Returns:

  EFI_SUCCESS - Operation succeeded.
  EFI_OUT_OF_RESOURCES - Memory allocation failed.
  Others      - Some failure happened.

--*/
;

EFI_STATUS
ResumeExecComd (
  VOID
  )
/*++

Routine Description:

  Resume SCT execution by executing "sct -c" in sct passive mode.

Arguments:

  None

Returns:

  EFI_SUCCESS - Operation succeeded.
  EFI_OUT_OF_RESOURCES - Memory allocation failed.
  Others      - Some failure happened.
  
--*/
;

#endif
