/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:

  BuildMenu.h

Abstract:

  This file provides the services to build a menu.

--*/

#ifndef _EFI_BUILD_MENU_H_
#define _EFI_BUILD_MENU_H_

//
// Includes
//

#include "Sct.h"
#include "Ui.h"

//
// Definitions
//

#define EFI_MENU_HEADER_ROWS        1
#define EFI_MENU_FOOTER_ROWS        2
#define EFI_MENU_DESCRIPTOR_COLS    20

#define EFI_CREATE_BY_PROTOCOL      0
#define EFI_CREATE_BY_DRIVER        1
#define EFI_CREATE_BY_FILENAME      2

//
// Prototypes (from BuildMenu.c)
//

EFI_STATUS
BuildMenuSystem (
  VOID
  );

EFI_STATUS
CreateStandardMenuPage (
  IN CHAR16                       *MenuItemHeaderText,
  OUT EFI_MENU_PAGE               **Page
  );

EFI_STATUS
DestroyMenuPage (
  IN EFI_MENU_PAGE                *Page
  );

EFI_STATUS
AddHotKey (
  IN CHAR16                       *HotKeyNameText,
  IN CHAR16                       *HotKeyDescText,
  IN UINT16                       ScanCode,
  IN CHAR16                       UnicodeChar,
  IN KEY_FUNCTION                 KeyFunc,
  IN BOOLEAN                      Visible,
  IN OUT EFI_MENU_PAGE            *Page
  );

EFI_STATUS
AddSimpleMenuItem (
  IN EFI_ITEM_TYPE                ItemType,
  IN CHAR16                       *ItemNameText,
  IN CHAR16                       *ItemDescText,
  IN VOID                         *Context,
  IN OUT EFI_MENU_PAGE            *Page
  );

EFI_STATUS
AddEditMenuItem (
  IN EFI_ITEM_TYPE                SubType,
  IN CHAR16                       *ItemNameText,
  IN CHAR16                       *ItemDescText,
  IN CHAR16                       *ItemValue,
  IN VOID                         *Context,
  IN OUT EFI_MENU_PAGE            *Page
  );

EFI_STATUS
AddCaseMenuItem (
  IN EFI_ITEM_TYPE                SubType,
  IN CHAR16                       *ItemNameText,
  IN CHAR16                       *ItemDescText,
  IN CHAR16                       *ItemValue,
  IN VOID                         *Context,
  IN OUT EFI_MENU_PAGE            *Page
  );

EFI_STATUS
AddMenuItemCheckValue (
  IN CHAR16                       *ItemNameText,
  IN UINTN                        MaxValue,
  IN UINTN                        MinValue,
  IN UINTN                        DefaultValue,
  IN OUT EFI_MENU_PAGE            *Page
  );

EFI_STATUS
AddPopupMenuItem (
  IN CHAR16                       *ItemNameText,
  IN CHAR16                       *ItemDescText,
  IN CHAR16                       *ItemValue,
  IN VOID                         *Context,
  IN OUT EFI_MENU_PAGE            *Page
  );

EFI_STATUS
AddComboListMenuItem (
  IN CHAR16                       *ItemNameText,
  IN CHAR16                       *ItemDescText,
  IN EFI_ITEM_VALUE_QUEUE         *ValueQueue,
  IN VOID                         *Context,
  IN OUT EFI_MENU_PAGE            *Page
  );

EFI_STATUS
AddSeparateLine (
  IN UINTN                        Number,
  IN OUT EFI_MENU_PAGE            *Page
  );

EFI_STATUS
InsertValueQueue (
  IN OUT EFI_ITEM_VALUE_QUEUE     **ValueQueue,
  IN CHAR16                       *StringValue,
  IN UINTN                        IntValue
  );

EFI_STATUS
DestroyValueQueue (
  IN EFI_ITEM_VALUE_QUEUE         *ValueQueue
  );

EFI_STATUS
AddScrollBar (
  IN OUT EFI_MENU_PAGE            *Page
  );

//
//protypes (from BuildMenuByFile.c)
//

EFI_STATUS
BuildMenuByFile (
  IN CHAR16                       *FileName,
  IN CHAR16                       *MenuItemHeaderText,
  OUT EFI_MENU_PAGE               **Page
  );

//
// Prototypes (from MainMenu.c)
//

EFI_STATUS
DisplayMainMenu (
  VOID
  );

//
// Prototypes (from ConfigMenu.c)
//

EFI_STATUS
DisplayConfigMenu (
  IN EFI_MENU_PAGE                *ParentPage
  );

//
// Prototypes (from CaseMgmtMenu.c)
//

EFI_STATUS
DisplayCaseMgmtMenu (
  IN EFI_MENU_PAGE                *ParentPage
  );

//
// Prototypes (from ProtocolMenu.c)
//

EFI_STATUS
DisplayProtocolMenu (
  IN EFI_MENU_PAGE                *ParentPage
  );

//
// Prototypes (from DriverMenu.c)
//

EFI_STATUS
DisplayDriverMenu (
  IN EFI_MENU_PAGE                *ParentPage
  );

//
// Prototypes (from FileSequenceMenu.c)
//

EFI_STATUS
DisplayFileNameMenu (
  IN EFI_MENU_PAGE                *ParentPage
  );

EFI_STATUS
DisplaySequenceMenu (
  IN EFI_MENU_PAGE                *ParentPage
  );

//
// Prototypes (from CaseMenu.c)
//

EFI_STATUS
DisplayCaseMenu (
  IN UINTN                        Type,
  IN EFI_MENU_PAGE                *ParentPage
  );

//
//prototypes (from ResultMgmtMenu.c)
//
EFI_STATUS
DisplayResultMgmtMenu (
  IN EFI_MENU_PAGE                *ParentPage
  );
//
//protypes (from UtilityMenu.c)
//
EFI_STATUS
DisplayUtilityMenu (
  IN EFI_MENU_PAGE                *ParentPage
  );

#endif
